# frozen_string_literal: true

module Jobshop # :nodoc:
  class Employee < ApplicationRecord
    belongs_to :organization, -> { readonly }, inverse_of: :employees

    belongs_to :user, inverse_of: :employee, optional: true

    has_many :department_assignments, inverse_of: :employee,
      class_name: "Jobshop::Department::Assignment"

    has_many :departments, through: :department_assignments

    has_many :employments, inverse_of: :employee,
      dependent: :restrict_with_exception

    has_many :rfq_line_assignments, through: :user,
      foreign_key: :assigned_to_id, class_name: "Jobshop::RFQ::Line::Assignment"

    validates :email,
      presence:   { if: :email_required? },
      format:     { if: :email_required?, with: /\A[^@\s]+@[^@\s]+\z/ },
      uniqueness: { if: :email_changed?, scope: :organization_id,
                    case_sensitive: false }

    scope :active, -> {
      joins(:employments).merge(Jobshop::Employment.active)
    }

    scope :inactive, -> {
      joins(:employments).merge(Jobshop::Employment.inactive)
    }

    accepts_nested_attributes_for :employments, allow_destroy: true,
      reject_if: :all_blank

    def active?
      !!employments.active.first
    end

    def inactive?
      !employments.active.first
    end

    def name
      @name = [ forename, surname ].join(" ")
    end

    private def email_required?
      true
    end
  end
end
