# frozen_string_literal: true

module Jobshop
  class Inspection::LimitCriterion < ApplicationRecord
    belongs_to :organization, -> { readonly }

    belongs_to :report, -> { readonly }

    has_one :criterion, -> (criterion) {
      where({
        organization_id: criterion.organization_id,
        report_id: criterion.report_id
      })
    }, as: :criterion, autosave: true, dependent: :destroy,
    inverse_of: :criterion

    belongs_to :report

    def criterion
      super || build_criterion
    end

    delegate :name, :name=, :position, :position=, to: :criterion

    validate :minimum_or_maximum_or_both,
      :minimum_less_than_maximum

    def minimum
      self[:minimum] && Unitwise(self[:minimum].truncate(4), self[:unit])
    end

    def minimum=(value)
      if value && value.respond_to?(:unit)
        self[:minimum] = value.value
        self[:unit] = value.unit
      else
        self[:minimum] = value
      end
    end

    def maximum
      self[:maximum] && Unitwise(self[:maximum].truncate(4), self[:unit])
    end

    def maximum=(value)
      if value && value.respond_to?(:unit)
        self[:maximum] = value.value
        self[:unit] = value.unit
      else
        self[:maximum] = value
      end
    end
=begin
    def specification
      @specification ||= [
        qualifier, [ minimum, maximum ].compact.join("/")
      ].compact.join("\n")
    end
=end
    def qualifier
      @qualifier ||= if minimum && !maximum
        "Minimum"
      elsif maximum && !minimum
        "Maximum"
      else
        nil
      end
    end

    def unbound?
      !minimum ^ !maximum
    end

    def bound?
      !unbound?
    end

    def pass?(value)
      !undersize?(value) && !oversize?(value)
    end

    def oversize?(value)
      value = Unitwise(value, self[:unit]) unless value.respond_to?(:unit)
      maximum && value > maximum || false
    end

    def undersize?(value)
      value = Unitwise(value, self[:unit]) unless value.respond_to?(:unit)
      minimum && value < minimum || false
    end

    # Generate a random value that has a 90% chance of being in spec.
    # TODO: This needs to go in the tests somewhere, not really in the model.
    # :nocov:
    def random
      value = if bound?
        rand(self[:minimum].to_f..self[:maximum].to_f) * 1.1
      elsif maximum
        rand(self[:maximum].to_f) * 1.1
      elsif minimum
        rand((self[:minimum].to_f * 0.8)..(self[:minimum].to_f * 2.0))
      end

      Unitwise(value, self[:unit])
    end
    # :nocov:

    private def minimum_or_maximum_or_both
      unless minimum || maximum
        errors[:base] << "must specify minimum, maximum or both"
      end
    end

    private def minimum_less_than_maximum
      if(minimum && maximum && minimum > maximum)
        errors[:base] << "minimum must be less than maximum"
      end
    end
  end
end
