# frozen_string_literal: true

module Jobshop
  class User < ApplicationRecord
    belongs_to :organization, -> { readonly }, inverse_of: :users

    has_one :employee, inverse_of: :user

    has_many :rfq_line_assignments, foreign_key: :assigned_to_id,
      class_name: "Jobshop::RFQ::Line::Assignment"

    has_many :role_assignments, inverse_of: :user

    has_many :roles, through: :role_assignments

    validates :email,
      presence:   { if: :email_required? },
      format:     { if: :email_required?, with: /\A[^@\s]+@[^@\s]+\z/ },
      uniqueness: { if: :email_changed?, scope: :organization_id,
                    case_sensitive: false }

    validates :encrypted_password, presence: true

    validates :password,
      presence:     { if: :password_required? },
      confirmation: { if: :password_required? }

    before_validation(on: :create) do
      self.organization ||= employee&.organization
    end

    devise :database_authenticatable, :recoverable

    def self.serialize_from_session(key, salt)
      record = where(id: key).eager_load(:organization).first
      record if record && record.authenticatable_salt == salt
    end

    def name
      employee&.name || id
    end

    private def password_required?
      !persisted? || !password.nil? || !password_confirmation.nil?
    end

    private def email_required?
      true
    end
  end
end
