# frozen_string_literal: true

require "jobshop/helpers/migration.rb"

class CreateCustomers < ActiveRecord::Migration[5.2]
  include Jobshop::Helpers::Migration

  def change
    create_customer_categories_table
    create_customers_table
    create_customer_contacts_table
  end

  private def create_customer_categories_table
    create_table_with_auditing :jobshop_customer_categories, primary_key: %i[
      organization_id id
    ] do |t|
      t.citext :name, null: false
      t.index %i[ organization_id name ], unique: true

      t.text :description
    end

    fk_organization :jobshop_customer_categories
  end

  private def create_customers_table
    create_table_with_auditing :jobshop_customers, primary_key: %i[
      organization_id id
    ] do |t|
      t.citext :name, null: false
      t.index %i[ organization_id name ], unique: true

      t.references :category, type: :uuid, null: false, foreign_key:
        { to_table: :jobshop_customer_categories }

      t.string :email_from_domain

      t.timestamps
    end

    fk_organization :jobshop_customers

    # Reverse index on email_from_domain allows for search on just a domain
    # given the entire address.
    #
    # example.com will be indexed as moc.elpmaxe
    add_index :jobshop_customers,
      "organization_id, reverse(lower(email_from_domain)) text_pattern_ops",
      name: :idx_jobshop_customers_reverse_address, unique: true

    foreign_key :jobshop_customers, :jobshop_customer_categories,
      { organization_id: :organization_id, category_id: :id }
  end

  private def create_customer_contacts_table
    create_table_with_auditing :jobshop_customer_contacts, primary_key: %i[
      organization_id customer_id id
    ] do |t|
      t.citext :email, null: false
      t.index %i[ organization_id email ], unique: true

      t.string :forename
      t.string :surname
    end

    fk_organization :jobshop_customer_contacts

    foreign_key :jobshop_customer_contacts, :jobshop_customers,
      { organization_id: :organization_id, customer_id: :id }
  end
end
