# frozen_string_literal: true

ENV["RAILS_ENV"] ||= "development"

require "benchmark"

def factory_paths
  [ Jobshop::Engine.root.join("spec/factories") ]
end

def do_with_spinner(name, &block)
  retval = nil
  title = "[:spinner] #{name.ljust(40, '.')}"
  spinner = TTY::Spinner.new(title, format: :dots)
  spinner.auto_spin
  time = Benchmark.realtime do
    retval = yield
  end
  spinner.success("done in #{(time * 1000).round(0)}ms")
  retval
end

do_with_spinner("Loading factories") do
  require "factory_bot_rails"
  FactoryBot.definition_file_paths = factory_paths
  FactoryBot.find_definitions if Rails.env.development?
end

@org = do_with_spinner("Creating Organization") do
  FactoryBot.create(:organization)
end

do_with_spinner("Creating Departments") do
  @accounting = FactoryBot.create(:department, {
    name: "Accounting", organization: @org
  })

  @engineering = FactoryBot.create(:department, {
    name: "Engineering", organization: @org
  })

  @hr = FactoryBot.create(:department, {
    name: "Human Resources", organization: @org
  })

  @manufacturing = FactoryBot.create(:department, {
    name: "Manufacturing", organization: @org
  })

  @quality = FactoryBot.create(:department, {
    name: "Quality", organization: @org
  })

  @sales = FactoryBot.create(:department, {
    name: "Sales", organization: @org
  })

  @sr = FactoryBot.create(:department, {
    name: "Sales", organization: @org
  })
end

do_with_spinner("Creating Employees") do
  @salesmen = { }
  @salesmen[:thomas] = FactoryBot.create(:salesman, organization: @org, departments: [ @sales ])
  @salesmen[:richard] = FactoryBot.create(:salesman, organization: @org, departments: [ @sales ])
  @salesmen[:harold] = FactoryBot.create(:salesman, organization: @org, departments: [ @sales ])

  @engineers = { }
  @engineers[:joe] = FactoryBot.create(:engineer, organization: @org, departments: [ @engineering ])
  @engineers[:jim] = FactoryBot.create(:engineer, organization: @org, departments: [ @engineering ])

  @hr1 = FactoryBot.create(:hr_specialist, organization: @org, departments: [ @hr ])
end

@places = do_with_spinner("Creating test Places") do
  FactoryBot.create_list(:place, 10, organization: @org)
end

@customers = do_with_spinner("Creating test customers") do
  FactoryBot.create_list(:customer, 10, organization: @org)
end

@products = do_with_spinner("Creating test Products") do
  FactoryBot.create_list(:product, 10, organization: @org)
end

do_with_spinner("Creating test Routers for Products") do
  @products.each do |product|
    rp = FactoryBot.create(:routing_process, {
      organization: @org, product: product
    })

    FactoryBot.create_list(:routing_step, rand(1..4), {
      organization: @org, routing_process: rp
    })
  end
end

do_with_spinner("Creating sample Orders") do
  10.times do
    ord = FactoryBot.create(:order, {
      organization: @org, customer: @customers.sample
    })

    FactoryBot.create_list(:order_line, rand(1..4), {
      organization: @org, order: ord, product: @products.sample
    })
  end
end

@widgets = do_with_spinner("Creating Collection of widgets") do
  wc = FactoryBot.create(:widget_collection, organization: @org)
  FactoryBot.create_list(:widget_thing, 50, organization: @org, collection: wc)
end

@widget_packs = do_with_spinner("Creating Collection of widget packs") do
  wpc = FactoryBot.create(:widget_pack_collection, organization: @org)
  FactoryBot.create_list(:widget_pack_thing, 50, {
    organization: @org, collection: wpc, widgets: @widgets
  })
end

do_with_spinner("Creating Inspection Reports") do
  5.times do
    report = FactoryBot.create(:inspection_report, organization: @org)
    Jobshop::Inspection::CRITERION_TYPES.each_with_index do |type, index|
      FactoryBot.create(type, {
        organization: @org, report: report, position: (index + 1)
      })
    end
  end
end

do_with_spinner("Creating RFQ Mailman") do
  @org.mailmen.create({
    address: "rfq@jobshop.io", handler_type: "Jobshop::Mailroom::RFQHandler"
  })
end

do_with_spinner("Creating sample RFQs") do
  10.times do
    case rand(0..100) % 3
    when 0
      FactoryBot.create :rfq, :verified, organization: @org,
        customer: @customers.sample
    when 1
      FactoryBot.create :rfq, :verifiable, organization: @org,
        customer: @customers.sample
    when 2
      FactoryBot.create :rfq, :unverifiable, organization: @org
    end
  end
end
