# frozen_string_literal: true

require "generators/jobshop/app/app_generator"

module Jobshop
  class CanaryBuilder < Jobshop::AppBuilder
    def readme
      # Do not generate README.md
    end

    # The db/migrate folder isn't created automatically.
    def db_migrate
      unless Dir.exist?("db/migrate")
        say_status :create, "db/migrate"
        Dir.mkdir("db/migrate")
      end
    end

    # Make a phony `bin/jobshop` file
    def bin_jobshop
      FileUtils.touch("bin/jobshop")
    end

    def expose_mailer_previews
      # Mailer previews don't really play nice with Engines so in the canary app
      # we create an initializer to expose them properly.
      initializer "jobshop_expose_mailer_previews.rb", <<~INITIALIZER
        if Rails.env.development?
          Rails.application.configure do
            config.action_mailer.preview_path = "\#{Jobshop::Engine.root}/spec/mailers"
          end
        end
      INITIALIZER
    end

    def localhost_tld_length
      # This allows us to easily use the localhost hostname in development.
      initializer "jobshop_tld_length.rb", <<~INITIALIZER
        if Rails.env.development? || Rails.env.test?
          Rails.application.configure do
            config.action_dispatch.tld_length = 0
          end
        end
      INITIALIZER
    end

    def dev_cert
      FileUtils.mkdir_p(".ssl")

      key_file = Pathname.pwd.join("../../.ssl/jobshop.test.key.pem")
      cert_file = Pathname.pwd.join("../../.ssl/jobshop.test.cert.pem")

      unless key_file.file? && cert_file.file?
        Dir.chdir("../..") do
          Jobshop::Generators::DevCertGenerator.start
        end
      end

      FileUtils.cp(key_file, ".ssl")
      FileUtils.cp(cert_file, ".ssl")
    end
  end

  module Generators
    class CanaryGenerator < Jobshop::Generators::AppGenerator
      def self.source_paths
        [ Jobshop::Generators::CanaryGenerator.source_root,
          Rails::Generators::AppGenerator.source_root ]
      end

      hide!

      def create_boot_file
        template "config/boot.rb.tt"
      end

      def finish_template
        build :bin_jobshop
        build :config_database_yml
        build :db_migrate
        build :expose_mailer_previews
        build :localhost_tld_length
        build :dev_cert
        super
      end

      def run_bundle
        super
        return if options[:pretend]
        bundle_command("exec rails db:drop:all")
        bundle_command("exec rails db:create")
        bundle_command("exec rails db:migrate")
      end

    protected
      def get_builder_class
        Jobshop::CanaryBuilder
      end
    end
  end
end
