# frozen_string_literal: true

require "base64"
require "open3"

require "generators/jobshop/canary/canary_generator"
require "generators/jobshop/dev_cert/dev_cert_generator"

require "jobshop/cli/base"

module Jobshop
  module CLI
    class Canary < Base

      desc "devcert", "Generate the self-signed key/certificate pair for HTTPS"
      def devcert
        Jobshop::Generators::DevCertGenerator.start
      end

      desc "server", "Runs the jobshop server with heroku local"
      def server
        Dir.chdir("spec/canary") do
          exec("heroku local")
        end
      end

      desc "reset [options]", "Recreate the development app"
      def reset(*args)
        FileUtils.rmtree(File.expand_path("spec/canary"))
        Jobshop::Generators::CanaryGenerator.start(
          args.unshift(File.expand_path("spec/canary")),
          jobshop_options: {
            database: "postgresql",
            skip_bootsnap: true,
            skip_bundle: true,
            skip_coffee: true,
            skip_gemfile: true,
            skip_git: true,
            skip_javascript: true,
            skip_listen: true,
            skip_sprockets: true,
            skip_test: true,
            skip_turbolinks: true
          }
        )
      end

      desc "seed", "Seed the Canary app"
      def seed
        require_environment!
        Jobshop::Engine.load_seed
      end

      desc "setup", "Setup the development environment"
      method_option :fast, aliases: "-f", type: :boolean, default: false,
        desc: "Don't recreate app/database. Only seed and reset password."
      method_option :password, aliases: "-p", type: :string,
        desc: "Set password for initial development user"
      method_option :verbose, aliases: "-v", type: :boolean, default: false,
        desc: "Show all output"
      def setup
        File.file?("jobshop.gemspec") || abort(<<~ABORT)
          Please run this command from the root of the Jobshop source tree.
        ABORT

        print_header
        quiet_command(<<~COMMAND, :check, "dependencies") unless options[:fast]
          gem install bundler --conservative
          bundle check || bundle install
          mkdir -p .git/safe
          mkdir -p spec/canary
        COMMAND

        Dir.chdir "spec/canary" do
          quiet_command(<<~COMMAND, :prepare, "environment")
            if psql -lqt | cut -d \\| -f 1 | grep -qw canary_development; then
              bundle exec rails db:environment:set RAILS_ENV=development
            fi
          COMMAND
        end unless options[:fast]

        quiet_command(<<~COMMAND, :install, "jobshop development app") unless options[:fast]
          jobshop reset
        COMMAND

        quiet_command(<<~COMMAND, :seed, "pseudorandom development data")
          jobshop seed
        COMMAND

        say_status :config, "environment template", :white
        FileUtils.cp(".env", "spec/canary/.env") if File.file?(".env")

        say_status :complete, "", :white
        say ""

        login_link = if options[:password]
          login_with_password_link(options[:password])
        else
          reset_password_link
        end

        message = <<~MESSAGE
          Thank you for installing #{set_color("Jobshop", :white, :bold)}.

          You can start the server by running:

            jobshop server

          Then open your browser and go to:

            #{login_link}

        MESSAGE

        say message
      end

      no_commands do
        def login_with_password_link(password)
          require_environment!

          resource.password = resource.password_confirmation = password
          resource.save

          Jobshop::Engine.routes.url_helpers.new_user_session_url(
            protocol: :https, host: link_host, port: 8443,
            user: { email: resource.email })
        end
        def reset_password_link
          require_environment!

          token = resource.send :set_reset_password_token

          @reset_password_link ||=
            Jobshop::Engine.routes.url_helpers.edit_user_password_url(
              host: link_host, reset_password_token: token)
        end

        private def resource
          @resource ||= Jobshop::Organization.order(created_at: :desc).first
            .users.order(created_at: :desc).first
        end
        
        private def link_host
          @link_host ||= Rails.env.development? ? "jobshop.test" : "HOSTNAME"
        end
      end
    end
  end
end
